////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include "config.h"

#include <kmessagebox.h>

#include <qstringlist.h>

#include "editormap.h"
#include "checkmap.h"


QString CheckMap::displayPos(int x,int y)
{
	QString pos;

	pos.sprintf("[X:%d,Y:%d]",x+1,y+1);
	return (pos);
}


QString CheckMap::strictCheckTransporter(Map *map,const QString& thismap,int x,int y)
{
	int found = 0;
	for (Transporter *t = map->transporters.first(); t!=0; t = map->transporters.next())
	{
		if (t->orig_x==x && t->orig_y==y) ++found;
	}

	if (found==0) return (QString("Map #%1 at %2: transporter with no origin")
			      .arg(thismap).arg(displayPos(x,y)));
	if (found>1) return (QString("Map #%1 at %2: more than one transporter origin")
			     .arg(thismap).arg(displayPos(x,y)));
	return (QString::null);
}


QStringList CheckMap::strictCheckTransporters(Map *map,const QString& thismap)
{
	QStringList r;

	for (Transporter *t = map->transporters.first(); t!=0; t = map->transporters.next())
	{
		QString num;
		num.setNum(map->transporters.at()+1);

		if (map->xy(t->orig_x,t->orig_y)!=Sprite_Transport)
		{
			r += QString("Map #%1 transporter #%2: origin at %3 is not a transporter")
				.arg(thismap).arg(num).arg(displayPos(t->orig_x,t->orig_y));
		}

		if (map->xy(t->dest_x,t->dest_y)!=Sprite_Empty &&
		    map->xy(t->dest_x,t->dest_y)!=Sprite_Repton)
		{
			r += QString("Map #%1 transporter #%2 (warning): destination at %3 is not empty")
				.arg(thismap).arg(num).arg(displayPos(t->dest_x,t->dest_y));
		}
	}

	return (r);
}


QStringList CheckMap::checkMap(QList<Map> maps)
{
	QStringList r;

	for (Map *map = maps.first(); map!=0; map = maps.next())
	{
		QString thismap;
		thismap.setNum(maps.at()+1);

		if (map->password=="")
		{
			r += QString("Map #%1: no password specified")
				.arg(thismap);
 		}

 		if (map->time_limit<=0)
		{
			r += QString("Map #%1: invalid time limit %2")
				.arg(thismap).arg(map->time_limit);
		}

		int num_reptons = 0;
		int num_diamonds = 0;
		int num_blips = 0;
		int num_cages = 0;
		int num_bombs = 0;
		int num_strongboxes = 0;
		int num_keys = 0;
		int num_rocks = 0;
		int num_eggs = 0;
		int num_crowns = 0;

		QString rt;
		for (int y = 0; y<map->height; ++y)
		{
			for (int x = 0; x<map->width; ++x)
			{
				switch (map->xy(x,y))
				{
case Sprite_Repton:			++num_reptons;
					break;

case Sprite_Diamond:			++num_diamonds;
					break;

case Sprite_Blip:			++num_blips;
					break;

case Sprite_Cage:			++num_cages;
					break;

case Sprite_Bomb:			++num_bombs;
					break;

case Sprite_Strongbox:			++num_strongboxes;
					break;

case Sprite_Key:			++num_keys;
					break;

case Sprite_Rock:			++num_rocks;
					break;

case Sprite_Egg:			++num_eggs;
					break;

case Sprite_Crown:			++num_crowns;
					break;

case Sprite_Transport:			rt = strictCheckTransporter(map,thismap,x,y);
					if (rt!=QString::null) r += rt;
					break;

default:				;		// Avoid warnings
				}
			}
		}

 		if (num_reptons==0) r += QString("Map #%1: no Repton start position")
					    .arg(thismap);
 		if (num_reptons>1) r += QString("Map #%1: more than one Repton start position")
					   .arg(thismap);

 		if (num_bombs==0) r += QString("Map #%1: there is no end level object")
					  .arg(thismap);

 		if (num_crowns==0) r += QString("Map #%1 (warning): there is no must collect object")
					   .arg(thismap);

 		if (num_cages>num_blips) r += QString("Map #%1: has %2 cages but only %3 spirits")
						 .arg(thismap)
						 .arg(num_cages).arg(num_blips);
 		if (num_cages<num_blips) r += QString("Map #%1 (warning): has %2 spirits but only %3 cages")
						 .arg(thismap)
						 .arg(num_blips).arg(num_cages);

 		if (num_strongboxes>0 && num_keys==0) r += QString("Map #%1: has %2 strongboxes but no keys")
							      .arg(thismap)
							      .arg(num_strongboxes);

		if (num_eggs>0 && num_rocks==0) r += QString("Map #%1 (warning): has %2 eggs but no rocks")
							.arg(thismap)
							.arg(num_eggs);

		r += strictCheckTransporters(map,thismap);
	}

	// Check for duplicated passwords
	QStrList pwds;
	for (Map *map = maps.first(); map!=0; map = maps.next())
	{
		pwds.append(map->password);
	}
	for (Map *map = maps.first(); map!=0; map = maps.next())
	{
		int n1 = maps.at();
		for (const char *s = pwds.at(n1+1); s!=0; s = pwds.next())
		{
			if (map->password==s)
			{
				int n2 = pwds.at();
				r += QString("Maps #%1 and #%2: have the same password")
					.arg(n1+1).arg(n2+1);
			}
		}
	}

	return (r);
}


CheckMap::Severity CheckMap::checkReport(QList<Map> maps,QWidget *parent,QString addtext,bool positive)
{
	QStringList r = checkMap(maps);
	if (r.empty())
	{
		if (positive) KMessageBox::information(parent,"The consistency check was successful");
		return (CheckMap::Ok);
	}


	QString s = QString("<qt>");
	s += "The consistency check failed for these reasons:<ul>";

	CheckMap::Severity sev = CheckMap::Warning;
	for (QStringList::iterator t = r.begin(); t!=r.end(); ++t)
	{
		s += "<li>";
		s += *t;
		s += "</li>";

		if ((*t).find("(warning)")==-1) sev = CheckMap::Fatal;
	}
	s += "</ul>";

	if (sev==CheckMap::Fatal && addtext!=QString::null) s += QString("%1.").arg(addtext);

	KMessageBox::sorry(parent,s);
	return (sev);
}
