////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include "config.h"

#include <qfile.h>
#include <qtstream.h>

#include "fileio.h"
#include "editormap.h"


Map::Map()
{
	data = 0;
}


Map::~Map()
{
	if (data!=0) delete [] data;
	deleteTransporters();
}


void Map::newMap(int xwidth, int xheight)
{
	width = xwidth;
	height = xheight;
	if (data) delete [] data;
	deleteTransporters();

	password = "none";
	time_limit = 300;

	data = new Sprite[width * height];

	for (int y = 0; y < height; ++y)
		for (int x = 0; x < width; ++x)
			xy(x, y) = Sprite_Empty;
}


bool Map::save(const QString& filename)
{
	QFile f(filename);
	if (!f.open(IO_WriteOnly))
	{
		FileIO::report("Cannot save map to '%1'",filename);
		return (false);
	}

	QTextStream t(&f);
	t << width << ' ' << height << ' ' << transporters.count() << ' ' << time_limit << '\n';
	t << password << '\n';

	for (Transporter *tr = transporters.first(); tr != 0;
	     tr = transporters.next()) {
		t << tr->orig_x << ' ' << tr->orig_y << ' '
		  << tr->dest_x << ' ' << tr->dest_y << '\n';
	}

	for (int y = 0; y < height; ++y) {
		for (int x = 0; x < width; ++x)
			t << (char)(((char)xy(x, y)) + 'A');
		t << '\n';
	}

	f.close();
	return (true);
}


bool Map::load(const QString& filename)
{
	if (data) delete [] data;
	deleteTransporters();

	QFile f(filename);
	if (!f.open(IO_ReadOnly))
	{
		FileIO::report("Cannot load map from '%1'",filename);
		return (false);
	}

	QTextStream t(&f);
	int num_transporters;
	t >> width >> height >> num_transporters >> time_limit;
	t >> password;

	while (num_transporters--)
	{
		int ox, oy, dx, dy;
		t >> ox >> oy >> dx >> dy;
		addTransporter(ox, oy, dx, dy);
	}

	data = new Sprite[width * height];

	for (int y = 0; y < height; ++y) {
		char c;
		for (int x = 0; x < width; ++x) {
			t >> c;
			xy(x, y) = (Sprite)(c - 'A');
		}
	}

	f.close();
	return (true);
}


void Map::addTransporter(int ox, int oy, int dx, int dy)
{
	transporters.append(new Transporter(ox, oy, dx, dy));
}


void Map::deleteTransporters()
{
	transporters.clear();
}
