////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#ifndef EDITORWINDOW_H
#define EDITORWINDOW_H

#include <kmainwindow.h>

#include "editormap.h"

class EditorWidget;
class SpriteEditor;
class MapEditor;
class KToggleAction;

class EditorWindow : public KMainWindow
{
	Q_OBJECT

public:
	EditorWindow(const char *name = 0);

	bool loadEpisode(const QString& path);
	bool loadSprites(const QString& path);
	void selectLevel(int level);

private:
	KToggleAction *strictAction;
	KToggleAction *alignAction;
	KToggleAction *spriteToggle;
	KToggleAction *mapToggle;
	KAction *realignAction;

	EditorWidget *view;
	SpriteEditor *sprite_window;
	MapEditor *map_window;

	QList<Map> maps;
	QPixmap *sprites;
	QString caption;
	bool modified;

	void updateMapsList();
	void updateTransportersList(int level);
	void alignWindows();
	void readOptions();
	void saveOptions();

public slots:
	void menu_strictCheck();
	void menu_save();
        void menu_spriteEditor();
        void menu_levelEditor();
	void menu_realign();

	void mapInsert();
	void mapRemove();
	void mapMoveUp();
	void mapMoveDown();
	void transporterInsert();
	void transporterChange();
	void transporterRemove();
	void selectedMap(int level);
	void changedPassword(const QString &s);
	void changedTime(int i);
	void changedSprite();
	void setModified(bool mod = true);
	void updateWindowStates();

protected:
	void moveEvent(QMoveEvent *e);
	void closeEvent(QCloseEvent *e);
	bool queryClose();
};

#endif // !EDITORWINDOW_H
