////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include "config.h"

#include <kapplication.h>
#include <kmessagebox.h>
#include <kcmdlineargs.h>
#include <kaboutdata.h>
#include <klocale.h>
#include <kstandarddirs.h>
#include <kglobal.h>

#include <stdlib.h>
#include <time.h>

#include "kreptoned.h"
#include "startupdialog.h"
#include "newepisodedialog.h"
#include "episodelistdialog.h"
#include "editorwindow.h"
#include "fileio.h"


const QString kreptonappname = "krepton";		// episodes stored here

static KCmdLineOptions options[] =
{
	{ 0, 0, 0 }
};


int newEpisode()
{
	QString name;
	NewEpisodeDialog d("New Episode");
	for (;;)
	{
		if (!d.exec()) return (-1);

		name = d.episodeName();
		if (name=="") KMessageBox::sorry(0,"Missing episode name");
		else
		{
			if (FileIO::locateEpisodeDir(name)==QString::null) break;
			KMessageBox::sorry(0,QString("An episode named '%1' (or similar) already exists.").arg(name));
		}
	}

	if (name=="") return (-1);

	const QString copy = d.copyFrom();
	QString srcpath = QString::null;
	QString destpath;

	kdDebug(0) << "main::newEpisode name='" << name << "' copy='" << copy << "'" << endl;

	srcpath = FileIO::locateEpisode(copy);
	if (srcpath==QString::null)
	{
		KMessageBox::error(0,QString("Source episode '%1' not found!").arg(copy));
		return (-1);
	}

	destpath = FileIO::copyEpisode(name,srcpath);
	if (destpath==QString::null) return (-1);

	EditorWindow *w = new EditorWindow("New Episode");
	if (!w->loadEpisode(destpath))
	{
		delete w;
		return (-1);
	}

	w->show();
	return (kapp->exec());
}


int editEpisode()
{
	EpisodeListDialog d("Edit Episode");
	if (!d.exec()) return (-1);
	const QString name = d.episodeName();

	QString path = FileIO::locateEpisode(name);
	if (path==QString::null)
	{
		KMessageBox::error(0,QString("Edit episode '%1' not found!").arg(path));
		return (-1);
	}

	EditorWindow *w = new EditorWindow("Edit Episode");
	if (!w->loadEpisode(path))
	{
		delete w;
		return (-1);
	}

	w->show();
	return (kapp->exec());
}


int removeEpisode()
{
	EpisodeListDialog d("Remove Episode");
	if (!d.exec()) return (-1);
	const QString name = d.episodeName();

	QString path = FileIO::locateEpisode(name,FileIO::Local);
	if (path==QString::null)
	{
		KMessageBox::error(0,QString("Remove episode '%1' not found!").arg(path));
		return (-1);
	}

	QString msg = QString("<qt>Are you sure you want to remove the episode '%1'<br>located at '%2'?")
		.arg(name).arg(path);
	if (KMessageBox::warningContinueCancel(0,msg,QString::null,
					       KGuiItem("Remove"))!=KMessageBox::Continue) return (-1);
	FileIO::removeEpisode(path);
	return (-1);
}


int main(int argc,char *argv[])
{
	KAboutData aboutData("kreptoned",I18N_NOOP("KRepton Editor"),
			     VERSION,
			     I18N_NOOP("Editor for KRepton episodes, maps and sprites"),
			     KAboutData::License_GPL,
			     "(c) 1998, Sandro Sigala\n(c) 2003, Jonathan Marten",0,0,
			     "jjm@keelhaul.demon.co.uk");

	aboutData.addAuthor("Sandro Sigala",I18N_NOOP("Original author"),"ssigala@globalnet.it");
	aboutData.addAuthor("Jonathan Marten",I18N_NOOP("KDE 3 conversion"),"jjm@keelhaul.demon.co.uk");
	aboutData.addCredit("Jasper Renow-Clarke",I18N_NOOP("BBC format and maps"),"cm5hdjjr@bs41.staffs.ac.uk");
	aboutData.addCredit("George Russell",I18N_NOOP("Feature suggestions"),"george.russell@clara.net");

	KCmdLineArgs::init(argc,argv,&aboutData);
	KCmdLineArgs::addCmdLineOptions(options);

	KApplication app;
 	if (!app.isRestored())
	{
		KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
// todo: decode mode and skip initial dialogue
		args->clear();
	}

	srand(time(NULL));

	KGlobal::dirs()->addResourceType("episodes",QString("share/apps/%1/episodes").arg(kreptonappname));

	StartupDialog d(aboutData.programName());
	for (;;)
	{
		int retval;

		d.updateState();
		switch (d.exec())
		{
case StartupDialog::NewEpisode:
			retval = newEpisode();
			if (retval!=-1) return (retval);
			break;

case StartupDialog::EditEpisode:
			retval = editEpisode();
			if (retval!=-1) return (retval);
			break;

case StartupDialog::RemoveEpisode:
			retval = removeEpisode();
			if (retval!=-1) return (retval);
			break;

default:		return (0);
		}
	}
}
