////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include "config.h"

#include <qframe.h>
#include <qpixmap.h>
#include <qpainter.h>

#include "kreptoned.h"
#include "editormap.h"
#include "mapgridframe.h"


MapGridFrame::MapGridFrame(QPixmap *xsprites,QWidget *parent,const char *name)
	: QFrame(parent,name)
{
	sprites = xsprites;
	setFrameStyle(Box | Raised);
	setLineWidth(2);
	setMidLineWidth(1);
	border = lineWidth() * 2 + midLineWidth();
	map = 0;
	map_sx = map_sy = 0;

	setSize(9,9);
	setMouseTracking(true);

	current_sprite = Sprite_Empty;
}


void MapGridFrame::setSize(int w,int h)
{
	map_width = w;
	map_height = h;

	const int deswidth = sprite_width*map_width + border*2;
	const int desheight = sprite_height*map_height + border*2;
	resize(deswidth,desheight);
}


void MapGridFrame::drawContents(QPainter *p)
{
	if (!map)
		return;
	for (int y = 0; y < map_height; ++y)
		for (int x = 0; x < map_width; ++x)
			p->drawPixmap(x * sprite_width + border,
				      y * sprite_height + border,
				      sprites[map->xy(map_sx + x, map_sy + y)]);
}


void MapGridFrame::mousePressEvent(QMouseEvent *e)
{
	if (e->button() == LeftButton || e->button() == RightButton) {
		if (e->x() < border || e->x() > width() - border
		    || e->y() < border || e->y() > height() - border)
			return;
		int x = (e->x() - border)/sprite_width + map_sx;
		int y = (e->y() - border)/sprite_height + map_sy;
		emit pressedButton(e->button(), x, y);
	}
}


void MapGridFrame::mouseMoveEvent(QMouseEvent *e)
{
	if (e->x() < border || e->x() > width() - border
	    || e->y() < border || e->y() > height() - border)
		return;

	int x = (e->x() - border)/sprite_width + map_sx;
	int y = (e->y() - border)/sprite_height + map_sy;

	if (e->state() & (LeftButton|RightButton))
	    emit pressedButton(e->state() & (LeftButton|RightButton), x, y);

	emit changedCoordinates(x, y);
}
