////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include "config.h"

#include <qvalidator.h>

#include <ctype.h>

#include "kreptoned.h"
#include "fileio.h"
#include "newepisodedialog.h"


class EpisodeValidator : public QValidator
{
public:
	EpisodeValidator(QWidget *parent = 0,const char *name = 0);
	State validate(QString& input,int& pos) const;
};


EpisodeValidator::EpisodeValidator(QWidget *parent,const char *name)
	: QValidator(parent,name)
{
}


EpisodeValidator::State EpisodeValidator::validate(QString& input,int&) const
{
	if (input=="") return (Intermediate);

	const char *s;
	for (s = input; *s!='\0'; ++s)
	{
		if (!isalnum(*s)) return (Invalid);
	}

	return (Acceptable);
}


NewEpisodeDialog::NewEpisodeDialog(const char *title,QWidget *parent, const char *name)
	: KDialogBase(parent,name,true,title,KDialogBase::Ok|KDialogBase::Cancel)
{
	w = new NewEpisodeWidget(this);
	setMainWidget(w);

	QStringList l;
	FileIO::loadEpisodeList(l,FileIO::All);
	for (QStringList::iterator it = l.begin(); it!=l.end(); ++it)
		w->templateComboBox->insertItem(*it);

	EpisodeValidator *validator = new EpisodeValidator(this);
	w->nameLineEdit->setValidator(validator);

	setFixedSize(calculateSize(w->size().width(),w->size().height()));
	adjustSize();

	connect(w->nameLineEdit,SIGNAL(textChanged(const QString&)),
		this,SLOT(slotNameChanged(const QString&)));
	slotNameChanged("");
	w->nameLineEdit->setFocus();
}


void NewEpisodeDialog::slotNameChanged(const QString &s)
{
	enableButtonOK(!s.isEmpty());
}
