////////////////////////////////////////////////////////////////////////////
//  
//  KReptonEd - editor for Repton levels for KDE
//  
//  Copyright (c) 1998 Sandro Sigala <sandro@sigala.it>
//  Copyright (c) 2003 Jonathan Marten <jjm@keelhaul.demon.co.uk>
//  
//  This program is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//  
//  It is distributed in the hope that it will be useful, but WITHOUT ANY
//  WARRANTY; without even the implied warranty of MERCHANTABILITY or
//  FITNESS FOR A PARTICULAR PURPOSE.  See the GNU General Public License
//  for more details.
//  
//  You should have received a copy of the GNU General Public License
//  along with this program; see the file COPYING for further details.
//  If not, write to the Free Software Foundation, Inc., 59 Temple Place -
//  Suite 330, Boston, MA 02111-1307, USA.
//
////////////////////////////////////////////////////////////////////////////

#include "config.h"

#include <qframe.h>
#include <qpixmap.h>
#include <qpainter.h>
#include <qimage.h>

#include "kreptoned.h"
#include "spritegridframe.h"


SpriteGridFrame::SpriteGridFrame(QPixmap *xsprites, QWidget *parent, const char *name)
	: QFrame(parent, name)
{
	sprites = xsprites;
	setFrameStyle(Box | Raised);
	setLineWidth(2);
	setMidLineWidth(1);
	border = lineWidth() * 2 + midLineWidth();
	resize(sprite_width * 8 + border*2-1,
	       sprite_height * 8 + border*2-1);
	setMouseTracking(true);

	current_sprite = Sprite_Empty;
}


void SpriteGridFrame::drawContents(QPainter *p)
{
	if (current_sprite<0 || current_sprite>=NUM_SPRITES) return;

	QImage img = sprites[current_sprite].convertToImage();
	for (int y = 0; y < sprite_height; ++y)
		for (int x = 0; x < sprite_width; ++x) {
			QColor c = img.pixel(x, y);
			p->fillRect(border + x * 8, border + y * 8,
				    7, 7, c);
		}
}


void SpriteGridFrame::mousePressEvent(QMouseEvent *e)
{
	if (e->button() == LeftButton || e->button() == RightButton) {
		if (e->x() < border || e->x() > width() - border
		    || e->y() < border || e->y() > height() - border)
			return;
		int x = (e->x() - border)/8;
		int y = (e->y() - border)/8;
		emit pressedButton(e->button(), x, y);
	}
}


void SpriteGridFrame::mouseMoveEvent(QMouseEvent *e)
{
	if (e->x() < border || e->x() > width() - border
	    || e->y() < border || e->y() > height() - border)
		return;

	int x = (e->x() - border)/8;
	int y = (e->y() - border)/8;

	if (e->state() & (LeftButton|RightButton))
		emit pressedButton(e->state() & (LeftButton|RightButton), x, y);

	emit changedCoordinates(x, y);
}
